'use strict';

var t = require('@babel/types');
var helperValidatorIdentifier = require('@babel/helper-validator-identifier');

const $stringLiteral = value => ({
  type: 'StringLiteral',
  value
});
const $identifier = name => ({
  type: 'Identifier',
  name
});
const $objectProperty = (key, value) => ({
  type: 'ObjectProperty',
  key,
  value,
  computed: false,
  shorthand: false,
  decorators: null
});
const $children = elements => 1 === elements.length ? elements[0] : {
  type: 'ArrayExpression',
  elements
};
const $pureAnnotation = () => [{
  type: 'CommentBlock',
  value: '#__PURE__'
}];

class ImportSpec {
  #path;
  #cache = new Map();
  #specifiers = [];
  constructor(path) {
    this.#path = path;
  }
  add(importName) {
    if (this.#cache.has(importName)) {
      return this.#cache.get(importName);
    }
    if (0 === this.#specifiers.length) {
      this.#path.unshiftContainer('body', {
        type: 'ImportDeclaration',
        specifiers: this.#specifiers,
        source: $stringLiteral('jsx-dom-runtime')
      });
    }
    const local = $identifier('_' + importName);
    this.#cache.set(importName, local);
    this.#specifiers.push({
      type: 'ImportSpecifier',
      local,
      imported: $identifier(importName)
    });
    return local;
  }
}

const svgTags = new Set(['altGlyph', 'altGlyphDef', 'altGlyphItem', 'animate', 'animateColor', 'animateMotion', 'animateTransform', 'circle', 'clipPath', 'color-profile', 'cursor', 'defs', 'desc', 'ellipse', 'feBlend', 'feColorMatrix', 'feComponentTransfer', 'feComposite', 'feConvolveMatrix', 'feDiffuseLighting', 'feDisplacementMap', 'feDistantLight', 'feFlood', 'feFuncA', 'feFuncB', 'feFuncG', 'feFuncR', 'feGaussianBlur', 'feImage', 'feMerge', 'feMergeNode', 'feMorphology', 'feOffset', 'fePointLight', 'feSpecularLighting', 'feSpotLight', 'feTile', 'feTurbulence', 'filter', 'font', 'font-face', 'font-face-format', 'font-face-name', 'font-face-src', 'font-face-uri', 'foreignObject', 'g', 'glyph', 'glyphRef', 'hkern', 'image', 'line', 'linearGradient', 'marker', 'mask', 'metadata', 'missing-glyph', 'mpath', 'path', 'pattern', 'polygon', 'polyline', 'radialGradient', 'rect', 'set', 'stop', 'svg', 'switch', 'symbol', 'text', 'textPath', 'tref', 'tspan', 'use', 'view', 'vkern']);
const htmlTags = new Set(['a', 'abbr', 'address', 'area', 'article', 'aside', 'audio', 'b', 'base', 'bdi', 'bdo', 'blockquote', 'body', 'br', 'button', 'canvas', 'caption', 'cite', 'code', 'col', 'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn', 'dialog', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'header', 'hgroup', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'kbd', 'label', 'legend', 'li', 'link', 'main', 'map', 'mark', 'math', 'menu', 'menuitem', 'meta', 'meter', 'nav', 'noscript', 'object', 'ol', 'optgroup', 'option', 'output', 'p', 'param', 'picture', 'pre', 'progress', 'q', 'rb', 'rp', 'rt', 'rtc', 'ruby', 's', 'samp', 'script', 'search', 'section', 'select', 'slot', 'small', 'source', 'span', 'strong', 'style', 'sub', 'summary', 'sup', 'table', 'tbody', 'td', 'template', 'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'track', 'u', 'ul', 'var', 'video', 'wbr']);
const mathmlTags = new Set(['annotation', 'annotation-xml', 'maction', 'math', 'merror', 'mfrac', 'mi', 'mmultiscripts', 'mn', 'mo', 'mover', 'mpadded', 'mphantom', 'mprescripts', 'mroot', 'mrow', 'ms', 'mspace', 'msqrt', 'mstyle', 'msub', 'msubsup', 'msup', 'mtable', 'mtd', 'mtext', 'mtr', 'munder', 'munderover', 'semantics', 'mfenced']);
const events = ['copy', 'cut', 'paste', 'compositionend', 'compositionstart', 'compositionupdate', 'focus', 'blur', 'change', 'beforeinput', 'input', 'reset', 'submit', 'invalid', 'load', 'error', 'keydown', 'keypress', 'keyup', 'abort', 'canplay', 'canplaythrough', 'durationchange', 'emptied', 'encrypted', 'ended', 'loadeddata', 'loadedmetadata', 'loadstart', 'pause', 'play', 'playing', 'progress', 'ratechange', 'seeked', 'seeking', 'stalled', 'suspend', 'timeupdate', 'volumechange', 'waiting', 'auxclick', 'click', 'contextmenu', 'dblclick', 'drag', 'dragend', 'dragenter', 'dragexit', 'dragleave', 'dragover', 'dragstart', 'drop', 'mousedown', 'mouseenter', 'mouseleave', 'mousemove', 'mouseout', 'mouseover', 'mouseup', 'select', 'touchcancel', 'touchend', 'touchmove', 'touchstart', 'pointerdown', 'pointermove', 'pointerup', 'pointercancel', 'pointerenter', 'pointerleave', 'pointerover', 'pointerout', 'gotpointercapture', 'lostpointercapture', 'scroll', 'scrollend', 'wheel', 'animationstart', 'animationend', 'animationiteration', 'transitionend', 'transitionstart', 'transitioncancel', 'transitionrun', 'enterpictureinpicture', 'leavepictureinpicture', 'resize', 'toggle', 'cancel', 'close', 'fullscreenchange', 'fullscreenerror'];
const DOMEvents = new Set(events.map(e => 'on' + e));
const eventTypes = new Set([...events, 'focusin', 'focusout']);
const attributes = new Set(['tabindex', 'inputmode', 'referrerpolicy', 'enterkeyhint', 'maxlength', 'minlength', 'itemprop', 'itemscope', 'itemtype', 'itemid', 'itemref', 'accesskey', 'elementtiming', 'fetchpriority', 'controlslist', 'dirname', 'formtarget', 'formmethod', 'formenctype', 'formaction', 'datetime', 'colspan', 'rowspan']);
const booleanAttributes = new Set(['async', 'autofocus', 'autoplay', 'controls', 'checked', 'crossorigin', 'capture', 'defer', 'disabled', 'contenteditable', 'formnovalidate', 'readonly', 'multiple', 'loop', 'required', 'hidden', 'open', 'selected', 'nomodule', 'novalidate', 'playsinline', 'reversed', 'inert', 'disablepictureinpicture', 'disableremoteplayback', 'popover', 'itemscope', 'declare', 'moz-opaque', 'ismap']);
const enumerated = new Set(['aria-atomic', 'aria-braillelabel', 'aria-brailleroledescription', 'aria-busy', 'aria-controls', 'aria-current', 'aria-describedby', 'aria-description', 'aria-details', 'aria-disabled', 'aria-dropeffect', 'aria-errormessage', 'aria-flowto', 'aria-grabbed', 'aria-haspopup', 'aria-hidden', 'aria-invalid', 'aria-keyshortcuts', 'aria-label', 'aria-labelledby', 'aria-live', 'aria-owns', 'aria-relevant', 'aria-roledescription', 'aria-autocomplete', 'aria-checked', 'aria-expanded', 'aria-level', 'aria-modal', 'aria-multiline', 'aria-multiselectable', 'aria-orientation', 'aria-placeholder', 'aria-pressed', 'aria-readonly', 'aria-required', 'aria-selected', 'aria-sort', 'aria-valuemax', 'aria-valuemin', 'aria-valuenow', 'aria-valuetext', 'aria-activedescendant', 'aria-colcount', 'aria-colindex', 'aria-colindextext', 'aria-colspan', 'aria-posinset', 'aria-rowcount', 'aria-rowindex', 'aria-rowindextext', 'aria-rowspan', 'aria-setsize', 'draggable', 'spellcheck']);
const svgDOMAttributes = new Map(Object.entries({
  accentHeight: 'accent-height',
  alignmentBaseline: 'alignment-baseline',
  arabicForm: 'arabic-form',
  baselineShift: 'baseline-shift',
  capHeight: 'cap-height',
  clipPath: 'clip-path',
  clipRule: 'clip-rule',
  colorInterpolation: 'color-interpolation',
  colorInterpolationFilters: 'color-interpolation-filters',
  colorProfile: 'color-profile',
  colorRendering: 'color-rendering',
  dominantBaseline: 'dominant-baseline',
  enableBackground: 'enable-background',
  fillOpacity: 'fill-opacity',
  fillRule: 'fill-rule',
  floodColor: 'flood-color',
  floodOpacity: 'flood-opacity',
  fontFamily: 'font-family',
  fontSize: 'font-size',
  fontSizeAdjust: 'font-size-adjust',
  fontStretch: 'font-stretch',
  fontStyle: 'font-style',
  fontVariant: 'font-variant',
  fontWeight: 'font-weight',
  glyphName: 'glyph-name',
  glyphOrientationHorizontal: 'glyph-orientation-horizontal',
  glyphOrientationVertical: 'glyph-orientation-vertical',
  horizAdvX: 'horiz-adv-x',
  horizOriginX: 'horiz-origin-x',
  imageRendering: 'image-rendering',
  letterSpacing: 'letter-spacing',
  lightingColor: 'lighting-color',
  markerEnd: 'marker-end',
  markerMid: 'marker-mid',
  markerStart: 'marker-start',
  overlinePosition: 'overline-position',
  overlineThickness: 'overline-thickness',
  paintOrder: 'paint-order',
  panose1: 'panose-1',
  pointerEvents: 'pointer-events',
  renderingIntent: 'rendering-intent',
  shapeRendering: 'shape-rendering',
  stopColor: 'stop-color',
  stopOpacity: 'stop-opacity',
  strikethroughPosition: 'strikethrough-position',
  strikethroughThickness: 'strikethrough-thickness',
  strokeDasharray: 'stroke-dasharray',
  strokeDashoffset: 'stroke-dashoffset',
  strokeLinecap: 'stroke-linecap',
  strokeLinejoin: 'stroke-linejoin',
  strokeMiterlimit: 'stroke-miterlimit',
  strokeOpacity: 'stroke-opacity',
  strokeWidth: 'stroke-width',
  textAnchor: 'text-anchor',
  textDecoration: 'text-decoration',
  textRendering: 'text-rendering',
  underlinePosition: 'underline-position',
  underlineThickness: 'underline-thickness',
  unicodeBidi: 'unicode-bidi',
  unicodeRange: 'unicode-range',
  unitsPerEm: 'units-per-em',
  vAlphabetic: 'v-alphabetic',
  vHanging: 'v-hanging',
  vIdeographic: 'v-ideographic',
  vMathematical: 'v-mathematical',
  vectorEffect: 'vector-effect',
  vertAdvY: 'vert-adv-y',
  vertOriginX: 'vert-origin-x',
  vertOriginY: 'vert-origin-y',
  wordSpacing: 'word-spacing',
  writingMode: 'writing-mode',
  xHeight: 'x-height',
  xlinkActuate: 'xlink:actuate',
  xlinkArcrole: 'xlink:arcrole',
  xlinkHref: 'href',
  xlinkRole: 'xlink:role',
  xlinkShow: 'xlink:show',
  xlinkTitle: 'xlink:title',
  xlinkType: 'xlink:type',
  xmlBase: 'xml:base',
  xmlLang: 'xml:lang',
  xmlSpace: 'xml:space'
}));
const htmlDOMAttributes = new Map(Object.entries({
  acceptCharset: 'accept-charset',
  className: 'class',
  httpEquiv: 'http-equiv',
  htmlFor: 'for',
  xlinkHref: 'href'
}));

const cache = new WeakMap();
const getObjectProperties = element => {
  if (cache.has(element)) {
    return cache.get(element);
  }
  const properties = [];
  element.attributes.push({
    type: 'JSXAttribute',
    name: {
      type: 'JSXIdentifier',
      name: '$'
    },
    value: {
      type: 'JSXExpressionContainer',
      expression: {
        type: 'ObjectExpression',
        properties
      }
    }
  });
  cache.set(element, properties);
  return properties;
};
const eventListener = (element, key, value) => {
  const properties = getObjectProperties(element);
  const name = key.name.toLowerCase();
  properties.push($objectProperty(eventTypes.has(name) ? $identifier(name) : helperValidatorIdentifier.isIdentifierName(key.name) ? $identifier(key.name) : $stringLiteral(key.name), value));
};

const convertJSXNamespacedName = node => $stringLiteral(node.namespace.name + ':' + node.name.name);
const buildProps = node => {
  const properties = node.openingElement.attributes.map(attr => {
    if ('JSXSpreadAttribute' === attr.type) {
      return {
        type: 'SpreadElement',
        argument: attr.argument
      };
    }
    const value = null === attr.value ? {
      type: 'BooleanLiteral',
      value: true
    } : 'JSXExpressionContainer' === attr.value.type ? 'JSXEmptyExpression' === attr.value.expression.type ? {
      type: 'NullLiteral'
    } : attr.value.expression : attr.value;
    if ('StringLiteral' === value.type) {
      value.value = value.value.replace(/\n\s+/g, ' ');
    }
    return $objectProperty('JSXNamespacedName' === attr.name.type ? convertJSXNamespacedName(attr.name) : helperValidatorIdentifier.isIdentifierName(attr.name.name) ? $identifier(attr.name.name) : $stringLiteral(attr.name.name), value);
  });
  const children = t.react.buildChildren(node);
  if (children.length > 0) {
    properties.push($objectProperty($identifier('children'), $children(children)));
  }
  return {
    type: 'ObjectExpression',
    properties
  };
};
const convertJSXIdentifier = node => {
  if ('JSXIdentifier' === node.type) {
    return $identifier(node.name);
  }
  return {
    type: 'MemberExpression',
    object: convertJSXIdentifier(node.object),
    property: $identifier(node.property.name),
    computed: false,
    optional: null
  };
};

const opts = {
  name: '_'
};
const isFunctionComponent = name => {
  const charCode = name.name.charCodeAt(0);
  return charCode >= 65 && charCode <= 90 || 36 === charCode || 95 === charCode;
};
let nsMap;
let importSpec;
const jsxTransform = {
  name: 'jsx-dom-runtime/babel-plugin-transform-jsx',
  visitor: {
    Program(path) {
      nsMap = new WeakMap();
      importSpec = new ImportSpec(path);
    },
    JSXFragment(path) {
      const children = t.react.buildChildren(path.node);
      if ('JSXElement' === path.parent.type) {
        if (children.length > 0) {
          path.replaceWith($children(children));
        } else {
          path.remove();
        }
      } else {
        path.replaceWith({
          type: 'CallExpression',
          callee: importSpec.add('Fragment'),
          arguments: children.length > 0 ? [$children(children)] : [],
          leadingComments: $pureAnnotation()
        });
      }
    },
    JSXElement: {
      enter(path) {
        const name = path.node.openingElement.name;
        if ('JSXNamespacedName' === name.type) {
          return;
        }
        if ('JSXMemberExpression' === name.type || isFunctionComponent(name)) {
          path.replaceWith({
            type: 'CallExpression',
            callee: convertJSXIdentifier(name),
            arguments: [buildProps(path.node)]
          });
        } else if (svgTags.has(name.name)) {
          nsMap.set(path, 'svgNs');
        } else if (mathmlTags.has(name.name)) {
          nsMap.set(path, 'mathmlNs');
        }
      },
      exit(path) {
        const name = path.node.openingElement.name;
        const props = buildProps(path.node);
        const noNs = props.properties.every(i => !t.isIdentifier(i.key, opts));
        if (noNs) {
          const importName = nsMap.get(path) ?? nsMap.get(path.findParent(p => 'JSXElement' === p.node.type));
          if (undefined !== importName) {
            props.properties.push($objectProperty($identifier('_'), importSpec.add(importName)));
          }
        }
        path.replaceWith({
          type: 'CallExpression',
          callee: importSpec.add('jsx'),
          arguments: ['JSXIdentifier' === name.type ? $stringLiteral(name.name) : convertJSXNamespacedName(name), props],
          leadingComments: $pureAnnotation()
        });
      }
    },
    JSXSpreadChild(path) {
      path.replaceWith(path.node.expression);
    },
    JSXAttribute(path) {
      const parent = path.parent;
      if ('JSXOpeningElement' !== parent.type || 'JSXIdentifier' !== parent.name.type) {
        return;
      }
      const tag = parent.name.name;
      const isHTMLElement = htmlTags.has(tag);
      const isSVGElement = svgTags.has(tag);
      const isMathMLElement = mathmlTags.has(tag);
      const isCustomElement = !(isSVGElement || isMathMLElement) && tag.includes('-', 1);
      if (!(isHTMLElement || isSVGElement || isCustomElement || isMathMLElement)) {
        return;
      }
      const node = path.node;
      const attr = node.name;
      if ('JSXNamespacedName' === attr.type) {
        if ('on' === attr.namespace.name && 'JSXExpressionContainer' === node.value.type && 'JSXEmptyExpression' !== node.value.expression.type) {
          eventListener(parent, attr.name, node.value.expression);
          path.remove();
        } else if (isCustomElement) {
          return;
        } else if ('href' === attr.name.name && 'xlink' === attr.namespace.name) {
          node.name = attr.name;
        }
        return;
      }
      if (isCustomElement) {
        return;
      }
      if (htmlDOMAttributes.has(attr.name)) {
        attr.name = htmlDOMAttributes.get(attr.name);
        return;
      }
      const attrName = attr.name.toLowerCase();
      if (DOMEvents.has(attrName)) {
        attr.name = attrName;
      } else if (booleanAttributes.has(attrName)) {
        attr.name = attrName;
        node.value ??= $stringLiteral('');
      } else if (enumerated.has(attrName) || attrName.startsWith('data-')) {
        const val = node.value;
        attr.name = attrName;
        if (null === val) {
          node.value = $stringLiteral('true');
        } else if ('JSXExpressionContainer' === val.type && 'BooleanLiteral' === val.expression.type) {
          node.value = $stringLiteral(val.expression.value ? 'true' : 'false');
        }
      } else if (isHTMLElement && attributes.has(attrName)) {
        attr.name = attrName;
      } else if (isSVGElement && svgDOMAttributes.has(attr.name)) {
        attr.name = svgDOMAttributes.get(attr.name);
      }
    }
  }
};

const preset = api => {
  api.assertVersion(7);
  return {
    plugins: [{
      manipulateOptions(_, parser) {
        parser.plugins.push('jsx');
      }
    }, jsxTransform]
  };
};

module.exports = preset;
